// This is a mutant program.
// Author : ysma

package com.mongodb.gridfs;


import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import com.mongodb.MongoException;
import org.bson.types.ObjectId;
import com.mongodb.BasicDBObject;
import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;


public class GridFS
{

    private static final java.util.logging.Logger LOGGER = Logger.getLogger( "com.mongodb.gridfs" );

    public static final int DEFAULT_CHUNKSIZE = 256 * 1024;

    public static final long MAX_CHUNKSIZE = (long) (3.5 * 1000 * 1000);

    public static final java.lang.String DEFAULT_BUCKET = "fs";

    public GridFS( com.mongodb.DB db )
    {
        this( db, DEFAULT_BUCKET );
    }

    public GridFS( com.mongodb.DB db, java.lang.String bucket )
    {
        _db = db;
        _bucketName = bucket;
        _filesCollection = _db.getCollection( _bucketName + ".files" );
        _chunkCollection = _db.getCollection( _bucketName + ".chunks" );
        try {
            if (_filesCollection.count() < 1000) {
                _filesCollection.ensureIndex( BasicDBObjectBuilder.start().add( "filename", 1 ).add( "uploadDate", 1 ).get() );
            }
            if (_chunkCollection.count() < 1000) {
                _chunkCollection.ensureIndex( BasicDBObjectBuilder.start().add( "files_id", 1 ).add( "n", 1 ).get(), BasicDBObjectBuilder.start().add( "unique", true ).get() );
            }
        } catch ( com.mongodb.MongoException e ) {
            LOGGER.info( String.format( "Unable to ensure indices on GridFS collections in database %s", db.getName() ) );
        }
        _filesCollection.setObjectClass( com.mongodb.gridfs.GridFSDBFile.class );
    }

    public  com.mongodb.DBCursor getFileList()
    {
        return getFileList( new com.mongodb.BasicDBObject() );
    }

    public  com.mongodb.DBCursor getFileList( com.mongodb.DBObject query )
    {
        return getFileList( query, new com.mongodb.BasicDBObject( "filename", 1 ) );
    }

    public  com.mongodb.DBCursor getFileList( com.mongodb.DBObject query, com.mongodb.DBObject sort )
    {
        return _filesCollection.find( query ).sort( sort );
    }

    public  com.mongodb.gridfs.GridFSDBFile find( org.bson.types.ObjectId id )
    {
        return findOne( id );
    }

    public  com.mongodb.gridfs.GridFSDBFile findOne( org.bson.types.ObjectId id )
    {
        return findOne( new com.mongodb.BasicDBObject( "_id", id ) );
    }

    public  com.mongodb.gridfs.GridFSDBFile findOne( java.lang.String filename )
    {
        return findOne( new com.mongodb.BasicDBObject( "filename", filename ) );
    }

    public  com.mongodb.gridfs.GridFSDBFile findOne( com.mongodb.DBObject query )
    {
        return _fix( _filesCollection.findOne( query ) );
    }

    public  java.util.List<GridFSDBFile> find( java.lang.String filename )
    {
        return find( filename, null );
    }

    public  java.util.List<GridFSDBFile> find( java.lang.String filename, com.mongodb.DBObject sort )
    {
        return find( new com.mongodb.BasicDBObject( "filename", filename ), sort );
    }

    public  java.util.List<GridFSDBFile> find( com.mongodb.DBObject query )
    {
        return find( query, null );
    }

    public  java.util.List<GridFSDBFile> find( com.mongodb.DBObject query, com.mongodb.DBObject sort )
    {
        java.util.List<GridFSDBFile> files = new java.util.ArrayList<GridFSDBFile>();
        com.mongodb.DBCursor c = null;
        try {
            c = _filesCollection.find( query );
            if (sort != null) {
                c.sort( sort );
            }
            while (c.hasNext()) {
                files.add( _fix( c.next() ) );
            }
        } finally 
{
            if (c != null) {
                c.close();
            }
        }
        return files;
    }

    protected  com.mongodb.gridfs.GridFSDBFile _fix( java.lang.Object o )
    {
        if (o == null) {
            return null;
        }
        if (!(o instanceof com.mongodb.gridfs.GridFSDBFile)) {
            throw new java.lang.RuntimeException( "somehow didn't get a GridFSDBFile" );
        }
        com.mongodb.gridfs.GridFSDBFile f = (com.mongodb.gridfs.GridFSDBFile) o;
        f._fs = this;
        return f;
    }

    public  void remove( org.bson.types.ObjectId id )
    {
        _filesCollection.remove( new com.mongodb.BasicDBObject( "_id", id ) );
        _chunkCollection.remove( new com.mongodb.BasicDBObject( "files_id", id ) );
    }

    public  void remove( java.lang.String filename )
    {
        remove( new com.mongodb.BasicDBObject( "filename", filename ) );
    }

    public  void remove( com.mongodb.DBObject query )
    {
        for (com.mongodb.gridfs.GridFSDBFile f: find( query )) {
            f.remove();
        }
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( byte[] data )
    {
        return createFile( new java.io.ByteArrayInputStream( data ), true );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.io.File f )
        throws java.io.IOException
    {
        return createFile( new java.io.FileInputStream( f ), f.getName(), true );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.io.InputStream in )
    {
        return createFile( in, null );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.io.InputStream in, boolean closeStreamOnPersist )
    {
        return createFile( in, null, closeStreamOnPersist );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.io.InputStream in, java.lang.String filename )
    {
        return new com.mongodb.gridfs.GridFSInputFile( this, in, filename );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.io.InputStream in, java.lang.String filename, boolean closeStreamOnPersist )
    {
        return new com.mongodb.gridfs.GridFSInputFile( this, in, filename, closeStreamOnPersist );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile( java.lang.String filename )
    {
        return new com.mongodb.gridfs.GridFSInputFile( this, filename );
    }

    public  com.mongodb.gridfs.GridFSInputFile createFile()
    {
        return new com.mongodb.gridfs.GridFSInputFile( this );
    }

    public  java.lang.String getBucketName()
    {
        return _bucketName;
    }

    public  com.mongodb.DB getDB()
    {
        return _db;
    }

    protected  com.mongodb.DBCollection getFilesCollection()
    {
        return _filesCollection;
    }

    protected  com.mongodb.DBCollection getChunksCollection()
    {
        return _chunkCollection;
    }

    protected final com.mongodb.DB _db;

    protected final java.lang.String _bucketName;

    protected final com.mongodb.DBCollection _filesCollection;

    protected final com.mongodb.DBCollection _chunkCollection;

}
